@@ -13,7 +13,7 @@ from bgeditor.dao import FFmpeg
 from bgeditor.dao.FFmpeg import create_suource_can_loop_path, create_loop, create_loop_audio, create_video_audio, merge_intro_outro
 from proglog import ProgressBarLogger
 from bgeditor.common.utils import get_dir
-import uuid
+import uuid, textwrap
 import math
 from moviepy.video.fx import make_loopable
 class MyBarLogger(ProgressBarLogger):
@@ -33,6 +33,7 @@ class MyBarLogger(ProgressBarLogger):
         except:
             pass
         return True
+
     def callback(self, **changes):
         # Every time the logger is updated, this function is called with
         # the `changes` dictionnary of the form `parameter: new value`.
@@ -53,6 +54,22 @@ class MyBarLogger(ProgressBarLogger):
             self.update_progress(self.bars[bar]['total'], self.bars[bar]['index'], rate)
             self.start_count_time_30= time.time()
 
+
+def create_lyric_outro(deezer_artist_id, lyric_outro_repeat,  arr_comp, time_o=5):
+    outro_sound = MusicHelper.create_lyric_outro_songs(deezer_artist_id, lyric_outro_repeat)
+
+    path_video = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + "-comp-outro-lyric-vid-audio.mp4")
+    tmp_path_composite_outro = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + "-comp-outro-lyric-vid.mp4")
+    tmp_clip = CompositeVideoClip(arr_comp).subclip(0, time_o)
+    tmp_clip.write_videofile(tmp_path_composite_outro, bitrate='4M', fps=24, codec='libx264', audio=False)
+    tmp_clip.close()
+
+    audio_compilation = AudioFileClip(outro_sound)
+    final_clip_path = create_loop(tmp_path_composite_outro, audio_compilation.duration)
+    utils.remove(tmp_path_composite_outro)
+    audio_compilation.close()
+    create_video_audio(final_clip_path, outro_sound, path_video)
+    return path_video
 def create_video(list_comp_data, path_video, job_id, mf_server):
     print('get list')
     intro_time=10
@@ -70,19 +87,28 @@ def create_video(list_comp_data, path_video, job_id, mf_server):
     max_duration = 0
     max_real_duration=3 # thoi gian cua 1 video
     is_compilation=False
+    is_lyric = False
+    is_lyric_outro = False
+    deezer_artist_id = None
     audio_compilation_path=None
     cnt_effect_vid=0
     introPath = None
     outroPath = None
+
     #check if videos are compilation
     for comp in arr_comps:
-        if comp.type=="compilation":
-            is_compilation=True
-            break
+        if comp.type == "lyric":
+            is_lyric = True
+            is_lyric_outro = comp.is_lyric_outro
+            if is_lyric_outro:
+                deezer_artist_id = comp.deezer_artist_id
+                lyric_outro_repeat = comp.lyric_outro_repeat
+        if comp.type == "compilation":
+            is_compilation = True
+            audio_compilation_path = comp.get_audio()
 
     for comp in arr_comps:
-        if comp.type=="compilation":
-            audio_compilation_path=comp.get_audio()
+        if comp.type == "compilation":
             continue
         if comp.type=="video":
             cnt_effect_vid += 1
@@ -119,7 +145,7 @@ def create_video(list_comp_data, path_video, job_id, mf_server):
         else:
             compTmp = comp.make()
             #cut tat ca comp nho hon 10s la intro
-            if is_compilation and comp.duration > 0 and comp.duration + comp.start_time <= intro_time:
+            if  comp.duration > 0 and comp.duration + comp.start_time <= intro_time:
                 if  comp.duration + comp.start_time > intro_time_real:
                     intro_time_real = comp.duration + comp.start_time
             else:
@@ -189,6 +215,9 @@ def create_video(list_comp_data, path_video, job_id, mf_server):
     final_vid = VideoFileClip(path_video)
     duration_f = final_vid.duration
     final_vid.close()
+    if is_lyric_outro:
+        lyricOutroPath = create_lyric_outro(deezer_artist_id, lyric_outro_repeat, arr_composite_no_intro)
+        path_video = merge_intro_outro(path_video, None, lyricOutroPath)
     path_video = merge_intro_outro(path_video, introPath, outroPath)
     for comp in arr_comps:
         comp.close()
@@ -196,6 +225,26 @@ def create_video(list_comp_data, path_video, job_id, mf_server):
 
 
 class Component:
+    def replace_code(self, ori_text):
+        if "@@" in ori_text:
+            try:
+                data = {"id": self.job_id, "ori_text": ori_text}
+                return requests.post(self.mf_server+"job/text/replace",data=data).text
+            except:
+                import traceback
+                traceback.print_exc()
+                return ""
+                pass
+        return ori_text
+    def get_max_font_size(self,font_url, text, width, font_size_want):
+        try:
+            data = {"font_url": font_url, "text": text,
+                    "width": width, "font_size_want": font_size_want}
+            obj = requests.post(self.mf_server + "text/font/max-size", data=data).json()
+            return obj['font_size'], obj['h_text']
+        except:
+            return None, None
+            pass
     def __init__(self, json_data):
         self.job_id=json_data['job_id']
         self.mf_server=json_data['mf_server']
@@ -325,6 +374,18 @@ class ElementComp(Component):
         return rs
 
 class TextComp(Component):
+    def calcWidthHigh(self):
+        cnt=self.text.count("\n")
+        ori_height = int(self.position['height'])
+        ori_y=int(self.position['y'])
+        self.position['height'] = ori_height
+        self.position['y']=ori_y
+        for i in range(cnt):
+            self.position['height'] += ori_height
+            self.position['y'] -= ori_y
+            if self.position['y'] < 0 :
+                self.position['y'] = 0
+                break
     def __init__(self, json_data):
         super().__init__(json_data)
         self.font_url = json_data['font_url']
@@ -338,10 +399,29 @@ class TextComp(Component):
         self.text = json_data['text']
         self.stroke_color = json_data['stroke_color']
         self.stroke_width = json_data['stroke_width']
+        self.is_replaced_text = False
+        if "@@" in self.text:
+            self.is_replaced_text = True
+            self.wrap_width = 50
+            if "wrap_width" in json_data and json_data['wrap_width']:
+                self.wrap_width = int(json_data['wrap_width'])
 
     def get_clip(self):
         self.font_path = cache_file(self.font_url)
-        rs = TextClip(txt= self.text, font = self.font_path.replace("\\","/"), fontsize=self.font_size, color=self.color,
+        if self.is_replaced_text:
+            self.text = self.replace_code(self.text)
+            wrapper = textwrap.TextWrapper(width=self.wrap_width)
+            self.text = wrapper.fill(text=self.text)
+            # self.calcWidthHigh()
+            font_size, h_text = self.get_max_font_size(self.font_url, self.text,
+                                                       int(self.position['width']), self.font_size)
+            if font_size:
+                self.font_size = font_size
+                self.position['y'] = self.position['y'] - (h_text - self.position['height'])
+                self.position['height'] = h_text
+                if self.position['y'] < 50:
+                    self.position['y']=50
+        rs = TextClip(txt = self.text, font = self.font_path.replace("\\","/"), fontsize=self.font_size, color=self.color,
                         bg_color = self.bg_color, size=(int(self.position['width']), int(self.position['height'])+10),
                         stroke_color = self.stroke_color, stroke_width=self.stroke_width)
         # if self.position['rotation'] != 0:
@@ -470,6 +550,13 @@ class LyricComp(Component):
         self.lyric_moving = json_data['lyric_moving']
         self.fade_in = json_data['fade_in']
         self.fade_out = json_data['fade_out']
+        self.is_lyric_outro=False
+        if "is_lyric_outro" in json_data:
+            self.is_lyric_outro = json_data['is_lyric_outro']
+            if "deezer_artist_id" in json_data:
+                self.deezer_artist_id = json_data['deezer_artist_id']
+            if "lyric_outro_repeat" in json_data:
+                self.lyric_outro_repeat = json_data['lyric_outro_repeat']
 
     def get_clip(self):
         self.lyric = Lyric(self.lyric_sync, self.font_url, self.font_size, self.color,
