@@ -7,7 +7,7 @@ import os
 import shutil
 
 def create_suource_can_loop_path(clip_path, is_delete = True, ext="mp4"):
-    return create_source_can_loop(get_dir('download'), clip_path, is_delete = is_delete, ext="mp4")
+    return create_source_can_loop(get_dir('download'), clip_path, is_delete = is_delete, ext=ext)
 def create_source_can_loop(dir_path, clip_path, is_delete = True, ext = "mp4" ):
     tmp_clip_path_z_resync = None
     try:
@@ -154,7 +154,7 @@ def create_video_audio(clip_path, audio_path, rs_path=None, is_del=True):
     return tmp_clip_path_z
 def convert_gif(video_path, is_del=True):
     tmp_clip_path_z = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-gif.avi')
-    cmd=f"ffmpeg -i \"{video_path}\"  -r 24 -c:v ffv1 \"{tmp_clip_path_z}\""
+    cmd=f"ffmpeg -i \"{video_path}\"  -c:v ffv1 \"{tmp_clip_path_z}\""
     os.system(cmd)
     if is_del:
         os.remove(video_path)
