@@ -14,6 +14,61 @@ def download_audio(url,ext='mp3'):
     with youtube_dl.YoutubeDL(ydl_opts) as ydl:
         ydl.download([url])
     return rs
+def create_lyric_outro_songs(deezer_artist_id, lyric_outro_repeat, retries=3):
+    res = requests.get(f"http://api-magicframe.automusic.win/lyric/deezer/artist/preview-song/{deezer_artist_id}/{lyric_outro_repeat}").json()
+    arr_song=[]
+    file_merg_path = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()))
+    file_merg = open(file_merg_path, "a")
+    final_clip_path = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-final.mp3')
+    for item in res:
+        tmp = download_file(item, ext='mp3')
+        try:
+            audio_test = AudioFileClip(tmp)
+            audioduration = audio_test.duration
+            audio_test.close()
+            if audioduration < 1:
+                continue
+        except:
+            continue
+            pass
+        tmp = normal_audio(tmp)
+        if not "coolbg_ffmpeg" in tmp:
+            tmp_clip_path = os.path.join(get_dir('coolbg_ffmpeg'),
+                                         str(uuid.uuid4()) + '-' + os.path.basename(tmp))
+            shutil.copyfile(tmp, tmp_clip_path)
+            os.remove(tmp)
+            tmp = tmp_clip_path
+        arr_song.append(tmp)
+        file_merg.write("file '%s'\n" % tmp)
+    file_merg.close()
+    if retries>0 and len(arr_song)!=lyric_outro_repeat:
+        retries-=1
+        os.remove(file_merg_path)
+        return create_lyric_outro_songs(deezer_artist_id, lyric_outro_repeat, retries=retries)
+
+    if len(arr_song) > 1:
+        cmd = "ffmpeg -y -f concat -safe 0 -i \"%s\" -codec copy \"%s\"" % (file_merg_path, final_clip_path)
+        os.system(cmd)
+        os.remove(file_merg_path)
+        for song in arr_song:
+            try:
+                os.remove(song)
+            except:
+                pass
+    else:
+        if len(arr_song) == 1:
+            return arr_song[0]
+    if len(arr_song) == 0:
+        return None
+    try:
+        audio_moviepy = AudioFileClip(final_clip_path)
+        if audio_moviepy.duration < 1:
+            final_clip_path = None
+        audio_moviepy.close()
+    except:
+        pass
+    return final_clip_path
+
 def create_compilation_songs(data, job_id, mf_server):
     #[{"type":3,"url":"","repeat":1}]
     #3: youtube_video
